;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; entry.asm - 32 Bit entry point for OS code
;; (c) 2001 Breckin Loggins
;; Copy Freely
;;

;; Alright kids, if you've been following along on the code so far you know
;; that right now we're at 0x10000 linear, we're in protected mode, and we
;; just JMPed from the bootsector

;; This kernel is a "technology test", we'll be adding things in as we want
;; to learn about them.  No attempt is made to be conservative with size or
;; aggressive with speed
;; 
;; Here's the memory map as it will look:

; 0x0000 - 0x0fff GDT (this gives 512 GDT Entries, fine for now)
; 0x1000 - 0x17ff IDT (gives us our max 256 IDT Entries)
; 0x1800 - 0xffff ~60K of Free Space (we'll use if for something later, or not)
;	       
; 0x10000 - 0x1ffff Kernel Code (all this stuff) 65K for that for now
; 0x20000 - 0x2ffff Stack Space (65K for it as well)
; 0x30000 - 0x3ffff Task State Segments (Enough for 614 tasks)
; 0x40000 - 0xb7fff Reserved for future use (IOW, will be filled out later)
; 0xb8000 - 0xb9fff Video Screen (80*25*2 Bytes)
; 0xba000 - 0xfffff Not Used right now
;
; 0x100000 - 0x700000 7MB of user space

;; Yes, space is wasted like crazy, but it's all for the sake of simplicity.
;; In the future, TSSs will be allocated in the first 103 (or so) bytes
;; of a processes memory and mapped into it.  Also, you'll noticed tables for 
;; memory management conspicuously missing. Well that was intentional, what
;; I'm going to do is separate memory into 4K "pages" (so they will be 
;; compatible with paging hardware when I turn it on) and 12 bytes out of 
;; each page will be used for memory management. 4 bytes for the back link, 
;; for bytes for the next link, and 4 bytes for any info we need to store.
;; I'm also assuming the user only has 8MB of ram, and only 7 of it will be
;; available for user programs.  When I emplement ram checking in the near 
;; future this will go away.  Also has I have to start mapping I/O addresses 
;; and such from the hardware a lot of stuff in the <1MB region will be moved.
;; The morals: 1) When you're the operating system you can do with memory
;; what you darn well like 2) Make things simple at first, just so you can 
;; figure out what the heck is happening.

BITS 32			       ; 32-Bit development
			
; Exported System Calls
GLOBAL _printReg
GLOBAL _start32
GLOBAL _delay

; Imported System Calls
EXTERN _loadKernelTSS
EXTERN _newTask

EXTERN _startSound
EXTERN _stopSound
EXTERN _beep

EXTERN _setPITFreq
EXTERN _delayMS

EXTERN lasttask

EXTERN _initInterrupts
EXTERN _int0
EXTERN _int1
EXTERN _int2
EXTERN _int3
EXTERN _int4
EXTERN _int5
EXTERN _int6
EXTERN _int7
EXTERN _int8
EXTERN _int9
EXTERN _intA
EXTERN _intB
EXTERN _intC
EXTERN _intD
EXTERN _intE
EXTERN _int10
EXTERN _int11
EXTERN _int12
EXTERN _int13
EXTERN _int20

_start32:
	
	
	mov byte [0xb8002], 'K'
	mov byte [0xb8004], 'R'
	mov byte [0xb8006], 'N'
	mov byte [0xb8008], 'L'
	
	call _initInterrupts		; Install interrupt handlers
	call _loadKernelTSS		; Install a first TSS for 
					; Multitasking
	push dword 100			; 10 MS time slices
	call _setPITFreq
	
	; We should be able to use interrupts now
	sti
	
	; Enable NMI
	in ax, 0x70
	and ax, 0x7f
	out 0x70, ax

	;; Start a new task
	push dword _task1
	push dword 0x2f000
	call _newTask			; Create the task
	
	;; Start another new task
	push dword _task2
	push dword 0x2a000
	call _newTask
	
	jmp 0x28:0000		; Get the ball rolling!

spin: jmp spin				; Used now to end the kernel task, 
				; Used in the future as a safety mechanism


;; Sample tasks
_task1:
	cli
	inc dword [task1ctr]
	mov byte [0xb8010], 'T'
	mov byte [0xb8012], '1'
	mov dword [lasttask], 0x20
	sti
.spin:
	
	mov dword [lasttask], 0x20
	inc dword [task1ctr]
	push dword [task1ctr]
	push byte 1
	call _printReg
	jmp .spin

_task2:
	cli
	inc dword [task2ctr]
	mov byte [0xb8016], 'T'
	mov byte [0xb8018], '2'
	mov dword [lasttask], 0x28
	sti
.spin:
	
	mov dword [lasttask], 0x28
	inc dword [task2ctr]
	push dword [task2ctr]
	push byte 2
	call _printReg
	push dword  20
	call _delayMS
	jmp .spin
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Internal Kernel Calls - DON'T USE 
;; OUTSIDE THE KERNEL

;; FUNCTION: _delay
;; ARGS: none
;; PURPOSE: The calling overhead, etc.. should be enough delay for all I/O

_delay:
	jmp .next
.next:
	ret
	
;; FUNCTION: _printReg
;; ARGUMENTS: ARG1 32-Bits the value to print
;;	      ARG2 8-Bits the row to print (0-24)
;; PURPOSE: Prints the 32 bit value of a register starting at 0xb8014

_printReg:
	push eax
	push ebx
	push ecx
	push edx
	
	;; Setup
	; Calculate the row
	mov eax, 160			; 80 Col * 2 Bytes
	xor ebx, ebx	
	mov bl, [esp+20]		; Row offset
	mul ebx
	add eax, 0xb8000		; Offset into framebuf
	mov edx, eax			; Save it
	
	mov eax, [esp+24]		; Character
	
	mov ecx, 0x10			; 8 Digits, 8 attributes

.continue
	mov ebx, eax
	and bl, 0x0F
	cmp bl, 0x9
	ja .hex
	add bl, 0x30
	jmp .print
	
.hex:
	add bl, 0x37

.print:	
	mov byte [edx + 14 + ecx], bl
	mov byte [edx + 15 + ecx], 0x07
	sub ecx, 2
	shr eax, 4
	
	cmp ecx, 0
	jnz .continue
	
	pop edx
	pop ecx
	pop ebx
	pop eax
	ret 8
; End of _printReg


;; Data Area

task1ctr
	dd 0
task2ctr
	dd 0

; The IDT Register, 6 Bytes
IDTR
	dw 0x90		 ; Size (18 entries times 8 Bytes/entry)
	dd 0x1000	 ; Base 0x1000 per our memory map


; The folowing variable keeps track of the number of GDT entries, it's 1 based
; and always includes the null selector at the beginning
num_gdt_entries		dd 3

; The fundamental task structure, the TSS
tss
	.backlink	dw 0	; A pointer to the previous TSS
	.resvd1		dw 0	; Reserved space
	.esp0		dd 0	; 32 Bits for ESP0
	.ss0		dw 0	; 16 Bits for SS0
	.resvd2		dw 0	; Reserved space
	.esp1		dd 0	; 32 Bits for ESP1
	.ss1		dw 0	; 16 Bits for SS1
	.resvd3		dw 0	; Reserved space
	.esp2		dd 0	; 32 Bits for ESP2
	.ss2		dw 0	; 16 Bits for SS2
	.resvd4		dw 0	; Reserved space
	.cr3		dd 0	; 32 Bits for CR3 (control register)
	.eip		dd 0	; 32 Bits for Instruction Pointer
	.eflags		dd 0	; 32 Bits for EFLAGS
	.eax		dd 0	; 32 Bits for EAX
	.ecx		dd 0	; 32 Bits for ECX
	.edx		dd 0	; 32 Bits for EDX
	.ebx		dd 0	; 32 Bits for EBX
	.esp		dd 0	; 32 Bits for ESP (stack pointer)
	.ebp		dd 0	; 32 Bits for EBP
	.esi		dd 0	; 32 Bits for ESI
	.edi		dd 0    ; 32 Bits for EDI
	.es		dw 0	; 16 Bits for ES
	.resvd5		dw 0	; Reserved space
	.cs		dw 0	; 16 Bits for CS
	.resvd6		dw 0	; Reserved space
	.ss		dw 0	; 16 Bits for SS (stack segment)
	.resvd7		dw 0	; Reserved space
	.ds		dw 0	; 16 Bits for DS
	.resvd8		dw 0	; Reserved space
	.fs		dw 0	; 16 Bits for FS
	.resvd9		dw 0	; Reserved space
	.gs		dw 0	; 16 Bits for GS
	.resvda		dw 0	; Reserved space
	.ldt		dw 0	; 16 Bits for Local Descriptor table
	.resvdb		dw 0    ; Reserved space
	.t		dw 0	; Only the first bit of .t is mutable
	.iomap		dw 0	; 16 Bits for the IO Map base address
tss_end		; So we can calculate how big we are (103 Bytes normally)

; As you can see, it's pretty straightforward, it just defines a structure that
; can hold the current hardware state of a segment at any given time.
; As this OS gets written, an OS specific task structure will be written to
; keep track of the software state as well

; The accompanying part, the task descriptor (goes into the GDT)
tss_desc
	.limit		dw 0	; How big is the TSS (must be > 103 dec)
	.base1500	dw 0	; Base address bits 15-00
	.base2316	db 0	; Base address bits 23-16
	.tdp		db 0	; Acronym: Type (3-0), 0 bit, DPL (6-5), P (7)
	.lag		db 0	; Acronym: Limit (3-0), AVL (4), 0 bit, 0 bit,
				;          G (7)
	.base3124	db 0	; Base address bits 31-24
tss_desc_end	; So we can calculate how big we are


