;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INT.ASM - Interrupt Handlers for SimpleOS
;; (c) 2001 Breckin Loggins
;; Copy Freely
;;

;; These functions are called whenever a particular interrupt fires off, 
;; obviously they don't do much right now, but they will later

; These are all GLOBAL
GLOBAL _initInterrupts
GLOBAL _int0
GLOBAL _int1
GLOBAL _int2
GLOBAL _int3
GLOBAL _int4
GLOBAL _int5
GLOBAL _int6
GLOBAL _int7
GLOBAL _int8
GLOBAL _int9
GLOBAL _intA
GLOBAL _intB
GLOBAL _intC
GLOBAL _intD
GLOBAL _intE
GLOBAL _int10
GLOBAL _int11
GLOBAL _int12
GLOBAL _int13
GLOBAL _int20

GLOBAL lasttask

; Imported System Calls
EXTERN _printReg
EXTERN _delay
EXTERN _beep

; Imported Data Items
EXTERN tickcount

;; Exception handlers

; Divide By Zero
_int0:
mov byte [0xb8010], 'D'
mov byte [0xb8011], 0x47
mov byte [0xb8012], '0'
mov byte [0xb8013], 0x47
hlt
iret

; Debug Trap
_int1:
mov byte [0xb8010], 'D'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'B'
mov byte [0xb8013], 0x47
hlt
iret

; NMI Interrupt
_int2:
mov byte [0xb8010], 'N'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'M'
mov byte [0xb8013], 0x47
hlt
iret

; Breakpoint
_int3:
mov byte [0xb8010], 'B'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'P'
mov byte [0xb8013], 0x47
hlt
iret

; Overflow
_int4:
mov byte [0xb8010], 'O'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'F'
mov byte [0xb8013], 0x47
hlt
iret

; BOUND Range Exceeded
_int5:
mov byte [0xb8010], 'B'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'R'
mov byte [0xb8013], 0x47
hlt
iret

; Invalid Opcode
_int6:
mov byte [0xb8010], 'U'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'D'
mov byte [0xb8013], 0x47
hlt
iret

; Device Not Available
_int7:
mov byte [0xb8010], 'N'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'M'
mov byte [0xb8013], 0x47
hlt
iret

; Double Fault
_int8:
mov byte [0xb8010], 'D'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'F'
mov byte [0xb8013], 0x47
hlt
iret

; Coprocessor Segment Overrun
_int9:
mov byte [0xb8010], 'C'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'P'
mov byte [0xb8013], 0x47
hlt
iret

; Invalid TSS
_intA:
mov byte [0xb8010], 'T'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'S'
mov byte [0xb8013], 0x47
hlt
iret

; Segment Not Present
_intB:
mov byte [0xb8010], 'N'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'P'
mov byte [0xb8013], 0x47
hlt
iret

; Stack-Segment Fault
_intC:
mov byte [0xb8010], 'S'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'S'
mov byte [0xb8013], 0x47
hlt
iret

; General Protection Fault
_intD:
mov byte [0xb8010], 'G'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'P'
mov byte [0xb8013], 0x47
pop eax
call _printReg
hlt
iret

; Page Fault
_intE:
mov byte [0xb8010], 'P'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'F'
mov byte [0xb8013], 0x47
hlt
iret

; Int F reserved

; Floating Point Error (Math Fault)
_int10:
mov byte [0xb8010], 'M'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'F'
mov byte [0xb8013], 0x47
hlt
iret

; Alignment Check
_int11:
mov byte [0xb8010], 'A'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'C'
mov byte [0xb8013], 0x47
hlt
iret

; Machine Check
_int12:
mov byte [0xb8010], 'M'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'C'
mov byte [0xb8013], 0x47
hlt
iret

; Streaming SIMD Extensions
_int13:
mov byte [0xb8010], 'X'
mov byte [0xb8011], 0x47
mov byte [0xb8012], 'F'
mov byte [0xb8013], 0x47
hlt
iret

;; Interrupt Handlers (0x20-0x27) (0x30-0x37)

; IRQ0 System Timer
_int20:
	; Do nothing here for now
	cli
	push eax
	add dword [tickcount], 1
	mov eax, [lasttask]
	call SendEOI
	cmp eax, 0x20
	je .jmp0x28
	cmp eax, 0x28
	je .jmp0x20
	jmp .continue
	
.jmp0x20:
	pop eax
	sti
	jmp 0x20:0x0000
	iret
	
.jmp0x28
	pop eax
	sti
	jmp 0x28:0x0000
	iret

.continue:
	pop eax
	sti
	iret

; IRQ1 Keyboard
_int21:
	; Do nothing here for now
	cli
	push eax
	mov eax, [lasttask]
	call SendEOI
	cmp eax, 0x20
	je .jmp0x28
	cmp eax, 0x28
	je .jmp0x20
	jmp .continue
	
.jmp0x20:
	pop eax
	sti
	jmp 0x20:0x0000
	iret
	
.jmp0x28:
	pop eax
	sti
	jmp 0x28:0x0000
	iret
.continue:
	pop eax
	sti
	iret
	
; IRQ2 PIC2 (not needed)
_int22:
	; Do nothing here
	cli
	call SendEOI
	sti
	iret

; IRQ3 Serial Port (com1 usually) 
_int23:
	; Do nothing here for now
	cli
	call SendEOI
	sti
	iret
	
; IRQ4 Serial Port (com2 usually)
_int24:
	; Do nothing here for now
	cli
	call SendEOI
	sti
	iret
	
; IRQ5 Reserved/Sound Card
_int25:
	; Do nothing here for now
	cli
	call SendEOI
	sti
	iret
	
; IRQ6 Floppy Disk Controller
_int26:
	; Do nothing here for now
	cli
	call SendEOI
	sti
	iret

; IRQ7 Parallel Port
_int27:
	; Do nothing here for now
	cli
	call SendEOI
	sti
	iret

; IRQ8 Real Time Clock
_int30:
	; Do nothing here for now
	cli
	call SendEOI2
	sti
	iret

; IRQ9 Redirect from PIC1
_int31:
	; Send EOI to PIC1
	cli
	call SendEOI
	; Send EOI to PIC2
	call SendEOI2
	sti
	iret

; IRQ10 Reserved
_int32:
	; Do nothing here for now
	cli
	call SendEOI
	call SendEOI2
	sti
	iret
	
; IRQ11 Reserved
_int33:
	; Do nothing here for now
	cli
	call SendEOI
	call SendEOI2
	sti
	iret

; IRQ12 PS/2 Mouse
_int34:
	; Show that mouse activity has occured
	cli
	mov byte [0xb8030], 'M'
	mov byte [0xb8031], 0x47
	call SendEOI
	call SendEOI2
	sti
	iret

; IRQ13 Math Co-processor
_int35:
	; Do nothing here for now
	cli
	call SendEOI
	call SendEOI2
	sti
	iret
	
; IRQ14 Hard Disk
_int36:
	; Do nothing here for now
	cli
	call SendEOI
	call SendEOI2
	sti
	iret

; IRQ15 Reserved
_int37:
	; Do nothing here for now
	cli
	call SendEOI
	call SendEOI2
	sti
	iret

;; FUNCTION: _initInterrupts
;; PURPOSE: Sets up the IDT for the exception handlers

_initInterrupts:
	
	push eax
	push ebx

	mov eax, 0x1000		; Offset into GDT
					
	; Int 0
	mov ebx, _int0
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
		
	; Int 1
	mov ebx, _int1
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 2
	mov ebx, _int2
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 3
	mov ebx, _int3
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 4
	mov ebx, _int4
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 5
	mov ebx, _int5
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 6
	mov ebx, _int6
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 7
	mov ebx, _int7
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 8
	mov ebx, _int8
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 9
	mov ebx, _int9
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int A
	mov ebx, _intA
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int B
	mov ebx, _intB
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int C
	mov ebx, _intC
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int D
	mov ebx, _intD
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int E
	mov ebx, _intE
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int F Resrved
	
	; Int 10
	mov ebx, _int10
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 11
	mov ebx, _int11
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 12
	mov ebx, _int12
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 13
	mov ebx, _int13
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
		
	;; Interrupt Handlers
	
	; Mov to 0x20 * 8 + 0x1000
	mov eax, 0x1100
	
	; Int 20
	mov ebx, _int20
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 21
	mov ebx, _int21
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 22
	mov ebx, _int22
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8	
	
	; Int 23
	mov ebx, _int23
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 24
	mov ebx, _int24
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 25
	mov ebx, _int25
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 26
	mov ebx, _int26
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 27
	mov ebx, _int27
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	mov eax, 0x1180		; Ints 30-37
	
	; Int 30
	mov ebx, _int30
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 31
	mov ebx, _int31
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 32
	mov ebx, _int32
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 33
	mov ebx, _int33
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 34
	mov ebx, _int34
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 35
	mov ebx, _int35
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 36
	mov ebx, _int36
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Int 37
	mov ebx, _int37
	mov word [eax], bx
	mov word [eax+2], 0x08
	mov word [eax+4], 0x8E00
	shr ebx, 16
	mov word [eax+6], bx
	add eax, 8
	
	; Now load the IDTR
	lidt [IDTR]
	call _delay
	
	; Redirect the IRQs
	call InitPic
	
	pop ebx
	pop eax
	
	ret
; End of _initInterrupts

;; Function: InitPic (local function)
;; ARGUMENTS: None
;; PURPOSE: Initializes the PIC and IRQ0-IRQ15 (which are redirected to 
;;	    INT 40-INT 55

InitPic:
	push eax
	push ebx
	
	;; Initialize PIC 1
	
	; Initialize the PIC
	mov al, 0x11
	out 0x20, al
	call _delay
	
	; Remap IRQs 0-7
	mov al, 0x20		; IRQ0 starts at INT 32 (dec)
	out 0x21, al
	call _delay

	mov al, 0x4		; Slave attached to IRQ2
	out 0x21, al
	call _delay
	
	mov al, 0x01		; Last command byte
	out 0x21, al
	call _delay
	
	;; Initialize PIC 2
	
	; Initialize the PIC
	mov al, 0x11
	out 0xa0, al
	call _delay
	
	; Remap IRQs 0-7
	mov al, 0x30		; IRQ8 starts at INT 48 (dec)
	out 0xa1, al
	call _delay
	
	mov al, 0x2		
	out 0xa1, al
	call _delay
	
	mov al, 0x01		; Last command byte
	out 0xa1, al
	call _delay

	;; Unmask all interrupts
	xor al, al
	out 0x21, al
	out 0xa1, al
	
	
	
	pop ebx
	pop eax
	ret

;; FUNCTION: SendEOI (Sends end of interrupt signal to the PIC)
;; ARGUMENTS: none
;; PURPOSE: Just call this function at the end of every IRQ handler to signal
;;		to the PIC that it can service another IRQ

SendEOI:
	push eax
	mov al, 0x20		; EOI Signal
	out 0x20, al		; Send it!
	
	pop eax
	ret
; End SendEOI

;; FUNCTION: SendEOI2 (sends end of interrupt signal to PIC2)
;; ARGUMENTS: none
;; PURPOSE: Same as SendEOI, but for PIC2

SendEOI2:
	push eax
	
	mov al, 0x20		; EOI Signal
	out 0xa0, al		; Send it!
	
	pop eax
	ret
; End of SendEOI2

;; Data Area

lasttask
	dd 0

; The IDT Register, 6 Bytes
IDTR
	dw 0x800		; Size (256 entries times 8 Bytes/entry)
	dd 0x1000		; Base 0x1000 per our memory map
