;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; TIMER.ASM - PIT Code for SimpleOS
;; (c)2001 Breckin Loggins
;; Copy Freely
;;

BITS 32

; FUNCTION IMPORTS
EXTERN _printReg

; FUNCTION EXPORTS
GLOBAL _setPITFreq
GLOBAL _delayMS
GLOBAL tickcount

SECTION .text

;; FUNCTION: _setPITFreq
;; ARGUMENTS: Push the frequency in Hz on to the stack
;; PURPOSE: Set's the PIT Frequemcy, you'll get a timer interrupt Hz times 
;;		per second.  

_setPITFreq:

	push eax
	push ebx
	push edx
	
	xor edx, edx
	mov eax, 0x1234DD		; PIT Frequency
	mov ebx, [esp+16]
	div ebx			        ; Our frequency
		
	mov ebx, eax			; Save frequency
	xor eax, eax

	mov al, 0x36			; Command word
	out 0x43, al		
	
	mov ax, bx
	out 0x40, al			; New Frequency
	
	mov al, ah
	out 0x40, al
	
	pop edx
	pop ebx
	pop eax
	
	ret 4
; End _setPITFreq

;; FUNCTION: _delayMS
;; ARGUMENTS: Push the amount of MS you want to delay on the stack
;; ARGUMENTS: returns after the amount of MS have been reached

_delayMS:
	push eax
	push ebx
	push ecx
	
	mov eax, [tickcount]		; Get Current tickcount
	mov ecx, 10
	mul ecx				; Mul 10 to get ms
	mov ecx, [esp+16]		; Get arg
.continue:
	push eax			; Save early tickcount
	mov eax, [tickcount]		; Get current tickcount
	mov ebx, 10
	mul ebx				; Mul 10 to get ms
	mov ebx, eax
	pop eax				; Restore eax
	sub ebx, eax			; Subtract earlier value
	cmp ebx, ecx			; Are they equal yet?
	jb .continue			; Nope, one more time
	
	pop ecx
	pop eax
	pop ebx
	ret 4
; end _delayMS


; Data area

; Ticks since boot time
tickcount
		dd 0